@@ -67,7 +67,8 @@ namespace Latte {
 View::View(Plasma::Corona *corona, QScreen *targetScreen, bool byPassWM)
     : PlasmaQuick::ContainmentView(corona),
       m_contextMenu(new ViewPart::ContextMenu(this)),
-      m_effects(new ViewPart::Effects(this))
+      m_effects(new ViewPart::Effects(this)),
+      m_interface(new ViewPart::ContainmentInterface(this))
 {      
     //! needs to be created after Effects because it catches some of its signals
     //! and avoid a crash from View::winId() at the same time
@@ -167,7 +168,7 @@ View::~View()
         m_indicator->unloadIndicators();
     }
 
-    disconnect(m_corona, &Latte::Corona::availableScreenRectChangedFrom, this, &View::availableScreenRectChangedFrom);
+    disconnectSensitiveSignals();
     disconnect(containment(), SIGNAL(statusChanged(Plasma::Types::ItemStatus)), this, SLOT(statusChanged(Plasma::Types::ItemStatus)));
 
     qDebug() << "dock view deleting...";
@@ -205,6 +206,10 @@ View::~View()
         delete m_indicator;
     }
 
+    if (m_interface) {
+        delete m_interface;
+    }
+
     if (m_visibility) {
         delete m_visibility;
     }
@@ -225,7 +230,9 @@ void View::init()
     connect(this, &QQuickWindow::heightChanged, this, &View::heightChanged);
     connect(this, &QQuickWindow::heightChanged, this, &View::updateAbsoluteGeometry);
 
-    connect(m_corona, &Latte::Corona::availableScreenRectChangedFrom, this, &View::availableScreenRectChangedFrom);
+    //! used in order to disconnect it when it should NOT be called because it creates crashes
+    connect(this, &View::availableScreenRectChangedFrom, m_corona, &Latte::Corona::availableScreenRectChangedFrom);
+    connect(this, &View::availableScreenRegionChangedFrom, m_corona, &Latte::Corona::availableScreenRegionChangedFrom);
 
     connect(this, &View::byPassWMChanged, this, &View::saveConfig);
     connect(this, &View::isPreferredForShortcutsChanged, this, &View::saveConfig);
@@ -233,16 +240,16 @@ void View::init()
     connect(this, &View::typeChanged, this, &View::saveConfig);
 
     connect(this, &View::normalThicknessChanged, this, [&]() {
-        emit m_corona->availableScreenRectChangedFrom(this);
+        emit availableScreenRectChangedFrom(this);
     });
 
     connect(m_effects, &ViewPart::Effects::innerShadowChanged, this, [&]() {
-        emit m_corona->availableScreenRectChangedFrom(this);
+        emit availableScreenRectChangedFrom(this);
     });
     connect(m_positioner, &ViewPart::Positioner::onHideWindowsForSlidingOut, this, &View::hideWindowsForSlidingOut);
     connect(m_positioner, &ViewPart::Positioner::screenGeometryChanged, this, &View::screenGeometryChanged);
     connect(m_positioner, &ViewPart::Positioner::windowSizeChanged, this, [&]() {
-        emit m_corona->availableScreenRectChangedFrom(this);
+        emit availableScreenRectChangedFrom(this);
     });
 
     connect(m_contextMenu, &ViewPart::ContextMenu::menuChanged, this, &View::contextMenuIsShownChanged);
@@ -288,7 +295,11 @@ bool View::inDelete() const
 
 void View::disconnectSensitiveSignals()
 {
-    disconnect(m_corona, &Latte::Corona::availableScreenRectChangedFrom, this, &View::availableScreenRectChangedFrom);
+    disconnect(this, &View::availableScreenRectChangedFrom, m_corona, &Latte::Corona::availableScreenRectChangedFrom);
+    disconnect(this, &View::availableScreenRegionChangedFrom, m_corona, &Latte::Corona::availableScreenRegionChangedFrom);
+    disconnect(m_corona, &Latte::Corona::availableScreenRectChanged, this, &View::availableScreenRectChangedForViewParts);
+    disconnect(m_corona, &Latte::Corona::availableScreenRectChangedFrom, this, &View::availableScreenRectChangedFromSlot);
+
     setLayout(nullptr);
 
     if (m_windowsTracker) {
@@ -296,7 +307,7 @@ void View::disconnectSensitiveSignals()
     }
 }
 
-void View::availableScreenRectChangedFrom(View *origin)
+void View::availableScreenRectChangedFromSlot(View *origin)
 {
     if (m_inDelete || origin == this)
         return;
@@ -475,8 +486,8 @@ void View::updateAbsoluteGeometry(bool bypassChecks)
     //! this is needed in order to update correctly the screenGeometries
     if (visibility() && corona() && visibility()->mode() == Types::AlwaysVisible) {
         //! main use of BYPASSCKECKS is from Positioner when the view changes screens
-        emit m_corona->availableScreenRectChangedFrom(this);
-        emit m_corona->availableScreenRegionChangedFrom(this);
+        emit availableScreenRectChangedFrom(this);
+        emit availableScreenRegionChangedFrom(this);
     }
 }
 
@@ -1112,6 +1123,11 @@ ViewPart::Indicator *View::indicator() const
     return m_indicator;
 }
 
+ViewPart::ContainmentInterface *View::interface() const
+{
+    return m_interface;
+}
+
 ViewPart::Positioner *View::positioner() const
 {
     return m_positioner;
@@ -1260,6 +1276,25 @@ void View::deactivateApplets()
     }
 }
 
+bool View::appletIsExpandable(const int id)
+{
+    if (!containment()) {
+        return false;
+    }
+
+    for (const auto applet : containment()->applets()) {
+        if (applet->id() == id) {
+            PlasmaQuick::AppletQuickItem *ai = applet->property("_plasma_graphicObject").value<PlasmaQuick::AppletQuickItem *>();
+
+            if (ai) {
+                return (ai->preferredRepresentation() != ai->fullRepresentation());
+            }
+        }
+    }
+
+    return false;
+}
+
 void View::toggleAppletExpanded(const int id)
 {
     if (!containment()) {
