@@ -22,6 +22,7 @@
 #define VIEW_H
 
 // local
+#include "containmentinterface.h"
 #include "effects.h"
 #include "positioner.h"
 #include "visibilitymanager.h"
@@ -195,8 +196,9 @@ public:
 
     PlasmaQuick::ConfigView *configView();
 
-    ViewPart::Effects *effects() const;
+    ViewPart::Effects *effects() const;   
     ViewPart::Indicator *indicator() const;
+    ViewPart::ContainmentInterface *interface() const;
     ViewPart::Positioner *positioner() const;
     ViewPart::VisibilityManager *visibility() const;
     ViewPart::WindowsTracker *windowsTracker() const;
@@ -227,6 +229,7 @@ public slots:
     Q_INVOKABLE void setBlockHiding(bool block);
     Q_INVOKABLE void toggleAppletExpanded(const int id);
 
+    Q_INVOKABLE bool appletIsExpandable(const int id);
     Q_INVOKABLE bool mimeContainsPlasmoid(QMimeData *mimeData, QString name);
     Q_INVOKABLE bool tasksPresent();
 
@@ -288,8 +291,13 @@ signals:
     void availableScreenRectChangedForViewParts();
     void customPluginsChanged();
 
+    //! are used to trigger the Corona relevant signals and in that
+    //! way we can disable any such signaling all together, e.g. through disconnectSensitiveSignals()
+    void availableScreenRectChangedFrom(Latte::View *origin);
+    void availableScreenRegionChangedFrom(Latte::View *origin);
+
 private slots:
-    void availableScreenRectChangedFrom(View *origin);
+    void availableScreenRectChangedFromSlot(View *origin);
     void configViewCreatedFor(Latte::View *view);
     void hideWindowsForSlidingOut();
     void preferredViewForShortcutsChangedSlot(Latte::View *view);
@@ -356,6 +364,7 @@ private:
     QPointer<ViewPart::ContextMenu> m_contextMenu;
     QPointer<ViewPart::Effects> m_effects;
     QPointer<ViewPart::Indicator> m_indicator;
+    QPointer<ViewPart::ContainmentInterface> m_interface;
     QPointer<ViewPart::Positioner> m_positioner;
     QPointer<ViewPart::VisibilityManager> m_visibility;
     QPointer<ViewPart::WindowsTracker> m_windowsTracker;
